export const register = async (firstName, lastName, phoneNumber, password) => {
    const response = await fetch('https://carsharing.kolei.ru/auth/register',{
        method: "POST",
        headers:{
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({
            phone: phoneNumber,
            password: password,
            firstName: firstName,
            lastName: lastName
        })
    })

    if(response.ok){
        return true
    }
    else{
        return false
    }
}

export const login = async (phone, password) => {
    const response = await fetch('https://carsharing.kolei.ru/auth/login',{
        method: "POST",
        headers:{
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({
            phone: phone,
            password: password
        })
    })

    if(!response.ok){
        return null
    }
    else{
        const data = await response.json()
        return data
    }
}

export const loadUser = async (token) => {
    const response = await fetch('https://carsharing.kolei.ru/user', {
        headers: {
            'Authorization' : `Bearer ${token}`
        }
    })
    
    if(response.ok){
        const data = await response.json()
        return data
    }
}

export const loadUserValid = async (token) => {
    const response = await fetch('https://carsharing.kolei.ru/user', {
        headers: {
            'Authorization' : `Bearer ${token}`
        }
    })
    
    if(response.ok){
        const data = await response.json()
        return data.valid
    }
}

export const uploadImage = async (token, file) => {
    const formData = new FormData()
    formData.append("file", file)
    formData.append("token", token)

    try{
        const response = await fetch('https://carsharing.kolei.ru/user/photo',{
            method: "POST",
            body: formData
        })
        if(response.ok){
            const data = await loadUser(token)
            return data
        }
        else{
            console.error('Ошибка загрузки фото:', response.statusText)
        }
    }

    catch(error){
        console.error('Ошибка:', error)
    }
}

export const loadCars = async (token) => {
    const response = await fetch('https://carsharing.kolei.ru/cars',{
        method: "GET",
        headers:{
            'Authorization' : `Bearer ${token}`
        }
    })
    if(response.ok){
        const data = await response.json()
        return data
    }
}